<?php
/**
 * Эта модель предназначена для работы с записями о багах.
 *
 * @version 1.0
 * @link http://www.simplecoding.org
 * @author Стаценко Владимир <vova_33@gala.net>
 */
class MBug extends Model {

	function MBug() {
		parent::Model();
	}

	/**
	 * Ищет все баги и комментарии к ним. Баги возвращаются начиная с последнего добавленного.
	 * 
	 * @param $from - порядковый номер бага в списке результатов (это не id)
	 * @param $count - количество багов, которые нужно найти
	 * @return массив со всеми багами и комментариями к ним, FALSE - если ничего не найдено
	 */
	function getAllBugsWithComments($from, $count) {
		if (!is_integer((int)$from) || !is_integer((int)$count)) {
			return false;
		}
		$qGetAll = 'SELECT b.* , c.link, c.name AS category, cm.id AS c_id,'
			.' cm.uname AS c_uname, cm.description AS c_description,'
			.' cm.comment_date, cm.parent_id, cm.bug_id, cm.uemail AS c_uemail'
			.' FROM comments AS cm'
			.' RIGHT JOIN ('
			.' SELECT * FROM bugs ORDER BY bugs.bug_date DESC'
			.' LIMIT ?, ?'
			.' ) b ON b.id = cm.bug_id'
			.' LEFT JOIN categories AS c ON b.category_id = c.id';
		$res = $this->db->query($qGetAll, array((int)$from, (int)$count));
		if ($res->num_rows() <= 0) {
			return false;
		}
		return $res->result_array();
	}
	
	/**
	 * Ищет все баги (без комментариев). Баги возвращаются начиная с последнего добавленного.
	 * 
	 * @param $from - порядковый номер бага в списке результатов (это не id)
	 * @param $count - количество багов, которые нужно найти
	 * @return массив со всеми багами и комментариями к ним, FALSE - если ничего не найдено
	 */
	function getAllBugs($from, $count) {
		if (!is_integer((int)$from) || !is_integer((int)$count)) {
			return false;
		}
		$qGetAllBugs = 'SELECT b.* , c.link, c.name AS category'
			.' FROM bugs AS b'
			.' LEFT JOIN categories AS c ON b.category_id = c.id'
			.' ORDER BY b.bug_date DESC LIMIT ?, ?';
		$res = $this->db->query($qGetAllBugs, array((int)$from, (int)$count));
		if ($res->num_rows() <= 0) {
			return false;
		}
		return $res->result_array();
	}
	
	/**
	 * Возвращает общее количество записей о багах.
	 * Этот метод имеет смысл вызывать только после getTree
	 * 
	 * @return количество записей о багах
	 */
	function getBugsCount() {
		return $this->db->count_all('bugs');
	}
	
	/**
	 * Возвращает общее количество записей о багах для отдельной категории.
	 * Этот метод имеет смысл вызывать только после getTree
	 * 
	 * @param $category - id категории в базе данных
	 * @return количество записей о багах
	 */
	function getBugsCountByCategory($category) {
		$qCountRyCategory = 'SELECT COUNT(b.id) AS catsnum FROM bugs AS b'
			.' LEFT JOIN categories AS c ON b.category_id = c.id'
			.' WHERE c.link=?';
		$res = $this->db->query($qCountRyCategory, array($category));
		return $res->row()->catsnum;
	}
	
	/**
	 * Ищет все баги (без комментариев) в заданной категории.
	 * Баги возвращаются начиная с последнего добавленного.
	 * 
	 * @param $category - id категории в базе данных
	 * @param $from - порядковый номер бага в списке результатов (это не id)
	 * @param $count - количество багов, которые нужно найти
	 * @return массив со всеми багами в указанной категории 
	 * и комментарии к ним, FALSE - если ничего не найдено
	 */
	function getBugsByCategory($category, $from, $count) {
		if (!is_integer((int)$from) || !is_integer((int)$count)) {
			return false;
		}
		$qGetCategoryBugs = 'SELECT b.* , c.link, c.name AS category'
			.' FROM bugs AS b'
			.' LEFT JOIN categories AS c ON b.category_id = c.id'
			.' WHERE c.link=?'
			.' ORDER BY b.bug_date DESC LIMIT ?, ?';
		$res = $this->db->query($qGetCategoryBugs, array($category, (int)$from, (int)$count));
		if ($res->num_rows() <= 0) {
			return false;
		}
		return $res->result_array();
	}

	/**
	 * Возвращает информацию о выбранном баге и все, связанные с ним комментарии
	 * 
	 * @param $id - id бага в базе данных
	 * @return массив с данными о баге и комментариях к нему или FALSE если он не найден
	 */
	function getBug($id) {
		if (is_integer($id) || $id <= 0) {
			return false;
		}
		$qGetBug = 'SELECT b.*, c.link, c.name AS category,'
			.' cm.id AS c_id, cm.uname AS c_uname, cm.description AS c_description,'
			.' cm.comment_date, cm.parent_id, cm.bug_id, cm.uemail AS c_uemail'
			.' FROM bugs AS b'
			.' LEFT JOIN categories AS c ON b.category_id=c.id'
			.' LEFT JOIN comments AS cm ON b.id=cm.bug_id WHERE b.id=?';
		$res = $this->db->query($qGetBug, array($id));
		
		if ($res->num_rows() < 1) {
			return false;
		}
		return $res->result_array();
	}
	
	/**
	 * Создает запись о найденном баге в БД.
	 * 
	 * @param $bugData - данные сообщения о баге
	 * @return id созданного бага или FALSE в случае неудачи
	 */
	function addNewBug($bugData) {
		$qAddBug = 'INSERT INTO bugs (title, uname, category_id, description, bug_date, uemail)'
			.' VALUES (?, ?, ?, ?, NOW(), ?)';
		$res = $this->db->query($qAddBug, array(
						$bugData['title'],
						$bugData['uname'],
						$bugData['category_id'],
						$bugData['description'],
						$bugData['uemail']
		));
		if ($res) {
			return $this->db->insert_id();
		}
		return $res;
	}
	
	/**
	 * Удаляет запись о найденном баге из базы данных.
	 * 
	 * @param $id - id бага в базе данных, который нужно удалить
	 * @return TRUE - если баг удален, FALSE - в противном случае
	 */
	function delete($id) {
		if (is_integer($id) || $id <= 0) {
			return false;
		}
		$qDelBug = 'DELETE FROM bugs WHERE id=?';
		return $this->db->query($qDelBug, array($id));
	}
}
?>